package io.dekorate.config;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Object;
import java.lang.Boolean;

public class DekorateConfigBuilder extends DekorateConfigFluentImpl<DekorateConfigBuilder> implements VisitableBuilder<DekorateConfig,DekorateConfigBuilder> {

    DekorateConfigFluent<?> fluent;
    Boolean validationEnabled;

    public DekorateConfigBuilder() {
        this(true);
    }

    public DekorateConfigBuilder(Boolean validationEnabled) {
        this(new DekorateConfig(), validationEnabled);
    }

    public DekorateConfigBuilder(DekorateConfigFluent<?> fluent) {
        this(fluent, true);
    }

    public DekorateConfigBuilder(DekorateConfigFluent<?> fluent,Boolean validationEnabled) {
        this(fluent, new DekorateConfig(), validationEnabled);
    }

    public DekorateConfigBuilder(DekorateConfigFluent<?> fluent,DekorateConfig instance) {
        this(fluent, instance, true);
    }

    public DekorateConfigBuilder(DekorateConfigFluent<?> fluent,DekorateConfig instance,Boolean validationEnabled) {
        this.fluent = fluent; 
        fluent.withResources(instance.getResources()); 
        this.validationEnabled = validationEnabled; 
    }

    public DekorateConfigBuilder(DekorateConfig instance) {
        this(instance,true);
    }

    public DekorateConfigBuilder(DekorateConfig instance,Boolean validationEnabled) {
        this.fluent = this; 
        this.withResources(instance.getResources()); 
        this.validationEnabled = validationEnabled; 
    }

    public EditableDekorateConfig build() {
        EditableDekorateConfig buildable = new EditableDekorateConfig(fluent.getResources());
        return buildable;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        DekorateConfigBuilder that = (DekorateConfigBuilder) o;
        if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;
        
        if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
        return true;
    }

    public int hashCode() {
        return java.util.Objects.hash(fluent,  validationEnabled,  super.hashCode());
    }

}
