package io.dekorate.kubernetes.config;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Object;
import java.lang.Boolean;

public class ImageConfigurationBuilder extends ImageConfigurationFluentImpl<ImageConfigurationBuilder> implements VisitableBuilder<ImageConfiguration,ImageConfigurationBuilder> {

    ImageConfigurationFluent<?> fluent;
    Boolean validationEnabled;

    public ImageConfigurationBuilder() {
        this(true);
    }

    public ImageConfigurationBuilder(Boolean validationEnabled) {
        this(new ImageConfiguration(), validationEnabled);
    }

    public ImageConfigurationBuilder(ImageConfigurationFluent<?> fluent) {
        this(fluent, true);
    }

    public ImageConfigurationBuilder(ImageConfigurationFluent<?> fluent,Boolean validationEnabled) {
        this(fluent, new ImageConfiguration(), validationEnabled);
    }

    public ImageConfigurationBuilder(ImageConfigurationFluent<?> fluent,ImageConfiguration instance) {
        this(fluent, instance, true);
    }

    public ImageConfigurationBuilder(ImageConfigurationFluent<?> fluent,ImageConfiguration instance,Boolean validationEnabled) {
        this.fluent = fluent; 
        fluent.withProject(instance.getProject()); 
        fluent.withAttributes(instance.getAttributes()); 
        fluent.withRegistry(instance.getRegistry()); 
        fluent.withGroup(instance.getGroup()); 
        fluent.withName(instance.getName()); 
        fluent.withVersion(instance.getVersion()); 
        fluent.withImage(instance.getImage()); 
        fluent.withDockerFile(instance.getDockerFile()); 
        fluent.withAutoBuildEnabled(instance.getAutoBuildEnabled()); 
        fluent.withAutoPushEnabled(instance.getAutoPushEnabled()); 
        this.validationEnabled = validationEnabled; 
    }

    public ImageConfigurationBuilder(ImageConfiguration instance) {
        this(instance,true);
    }

    public ImageConfigurationBuilder(ImageConfiguration instance,Boolean validationEnabled) {
        this.fluent = this; 
        this.withProject(instance.getProject()); 
        this.withAttributes(instance.getAttributes()); 
        this.withRegistry(instance.getRegistry()); 
        this.withGroup(instance.getGroup()); 
        this.withName(instance.getName()); 
        this.withVersion(instance.getVersion()); 
        this.withImage(instance.getImage()); 
        this.withDockerFile(instance.getDockerFile()); 
        this.withAutoBuildEnabled(instance.getAutoBuildEnabled()); 
        this.withAutoPushEnabled(instance.getAutoPushEnabled()); 
        this.validationEnabled = validationEnabled; 
    }

    public io.dekorate.kubernetes.config.EditableImageConfiguration build() {
        EditableImageConfiguration buildable = new EditableImageConfiguration(fluent.getProject(),fluent.getAttributes(),fluent.getRegistry(),fluent.getGroup(),fluent.getName(),fluent.getVersion(),fluent.getImage(),fluent.getDockerFile(),fluent.getAutoBuildEnabled(),fluent.getAutoPushEnabled());
        return buildable;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        ImageConfigurationBuilder that = (ImageConfigurationBuilder) o;
        if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;
        
        if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
        return true;
    }

    public int hashCode() {
        return java.util.Objects.hash(fluent,  validationEnabled,  super.hashCode());
    }

}
