package io.dekorate.kubernetes.config;

import java.lang.StringBuffer;
import java.lang.StringBuilder;
import io.fabric8.kubernetes.api.builder.BaseFluent;
import java.util.ArrayList;
import java.util.Collection;
import java.lang.Object;
import java.util.List;
import java.lang.String;
import java.lang.Boolean;

public class LabelFluentImpl<A extends LabelFluent<A>> extends io.fabric8.kubernetes.api.builder.BaseFluent<A> implements LabelFluent<A> {

    private String key;
    private String value;
    private List<String> kinds = new ArrayList<String>();

    public LabelFluentImpl() {
    }

    public LabelFluentImpl(io.dekorate.kubernetes.config.Label instance) {
        this.withKey(instance.getKey()); 
        this.withValue(instance.getValue()); 
        this.withKinds(instance.getKinds()); 
    }

    public String getKey() {
        return this.key;
    }

    public A withKey(String key) {
        this.key=key; return (A) this;
    }

    public Boolean hasKey() {
        return this.key != null;
    }

    public A withNewKey(String arg1) {
        return (A)withKey(new String(arg1));
    }

    public A withNewKey(StringBuilder arg1) {
        return (A)withKey(new String(arg1));
    }

    public A withNewKey(StringBuffer arg1) {
        return (A)withKey(new String(arg1));
    }

    public String getValue() {
        return this.value;
    }

    public A withValue(String value) {
        this.value=value; return (A) this;
    }

    public Boolean hasValue() {
        return this.value != null;
    }

    public A withNewValue(String arg1) {
        return (A)withValue(new String(arg1));
    }

    public A withNewValue(StringBuilder arg1) {
        return (A)withValue(new String(arg1));
    }

    public A withNewValue(StringBuffer arg1) {
        return (A)withValue(new String(arg1));
    }

    public A withKinds(String... kinds) {
        if (this.kinds != null) {this.kinds.clear();}
        if (kinds != null) {for (String item :kinds){ this.addToKinds(item);}} return (A) this;
    }

    public String[] getKinds() {
        //This needs to work with primitives, so we use arrays.
        int size = kinds != null ? kinds.size() : 0;
        String[] result = new String[size];
        if (size == 0) {
           return result;
        }
        int index = 0;
        for (String item : kinds) {
            result[index++]=item;
        }
        return result;
    }

    public A addToKinds(int index,String item) {
        if (this.kinds == null) {this.kinds = new ArrayList<String>();}
        this.kinds.add(index, item);
        return (A)this;
    }

    public A setToKinds(int index,String item) {
        if (this.kinds == null) {this.kinds = new ArrayList<String>();}
        this.kinds.set(index, item); return (A)this;
    }

    public A addToKinds(String... items) {
        if (this.kinds == null) {this.kinds = new ArrayList<String>();}
        for (String item : items) {this.kinds.add(item);} return (A)this;
    }

    public A addAllToKinds(Collection<String> items) {
        if (this.kinds == null) {this.kinds = new ArrayList<String>();}
        for (String item : items) {this.kinds.add(item);} return (A)this;
    }

    public A removeFromKinds(String... items) {
        for (String item : items) {if (this.kinds!= null){ this.kinds.remove(item);}} return (A)this;
    }

    public A removeAllFromKinds(Collection<String> items) {
        for (String item : items) {if (this.kinds!= null){ this.kinds.remove(item);}} return (A)this;
    }

    public Boolean hasKinds() {
        return kinds != null && !kinds.isEmpty();
    }

    public A addNewKind(String arg1) {
        return (A)addToKinds(new String(arg1));
    }

    public A addNewKind(StringBuilder arg1) {
        return (A)addToKinds(new String(arg1));
    }

    public A addNewKind(StringBuffer arg1) {
        return (A)addToKinds(new String(arg1));
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        LabelFluentImpl that = (LabelFluentImpl) o;
        if (key != null ? !key.equals(that.key) :that.key != null) return false;
        if (value != null ? !value.equals(that.value) :that.value != null) return false;
        if (kinds != null ? !kinds.equals(that.kinds) :that.kinds != null) return false;
        return true;
    }

    public int hashCode() {
        return java.util.Objects.hash(key,  value,  kinds,  super.hashCode());
    }

}
