package io.dekorate.kubernetes.config;

import io.dekorate.project.Project;
import java.lang.Object;
import io.dekorate.kubernetes.annotation.AccessMode;
import java.lang.String;
import java.util.Map;
import io.sundr.builder.annotations.Buildable;

@Buildable(builderPackage = "io.fabric8.kubernetes.api.builder") public class PersistentVolumeClaimConfig extends ApplicationConfiguration {

    private int size;
    private String unit;
    private String storageClass;
    private AccessMode accessMode;
    private io.dekorate.kubernetes.config.Label[] matchLabels = new io.dekorate.kubernetes.config.Label[0];

    public PersistentVolumeClaimConfig() {
    }

    public PersistentVolumeClaimConfig(Project project,Map<ConfigKey,Object> attributes,String partOf,String name,String version,int size,String unit,String storageClass,AccessMode accessMode,io.dekorate.kubernetes.config.Label[] matchLabels) {
        super(project, attributes, partOf, name, version);
        this.size = size;
        this.unit = unit;
        this.storageClass = storageClass;
        this.accessMode = accessMode;
        this.matchLabels = matchLabels != null ? matchLabels : new io.dekorate.kubernetes.config.Label[0];
    }

    public int getSize() {
        return this.size;
    }

    public String getUnit() {
        return this.unit;
    }

    public String getStorageClass() {
        return this.storageClass;
    }

    public AccessMode getAccessMode() {
        return this.accessMode;
    }

    public io.dekorate.kubernetes.config.Label[] getMatchLabels() {
        return this.matchLabels;
    }

    public static PersistentVolumeClaimConfigBuilder newPersistentVolumeClaimConfigBuilder() {
        return new io.dekorate.kubernetes.config.PersistentVolumeClaimConfigBuilder();
    }

    public static PersistentVolumeClaimConfigBuilder newPersistentVolumeClaimConfigBuilderFromDefaults() {
        return new io.dekorate.kubernetes.config.PersistentVolumeClaimConfigBuilder().withSize(1).withUnit("Gi").withStorageClass("standard").withAccessMode(io.dekorate.kubernetes.annotation.AccessMode.ReadWriteOnce);
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        PersistentVolumeClaimConfig that = (PersistentVolumeClaimConfig) o;
        if (size != that.size) return false;
        if (unit != null ? !unit.equals(that.unit) :that.unit != null) return false;
        if (storageClass != null ? !storageClass.equals(that.storageClass) :that.storageClass != null) return false;
        if (accessMode != null ? !accessMode.equals(that.accessMode) :that.accessMode != null) return false;
        if (matchLabels != null ? !matchLabels.equals(that.matchLabels) :that.matchLabels != null) return false;
        return true;
    }

    public int hashCode() {
        return java.util.Objects.hash(size,  unit,  storageClass,  accessMode,  matchLabels,  super.hashCode());
    }

}
