package io.dekorate.kubernetes.config;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.StringBuilder;
import io.fabric8.kubernetes.api.builder.Nested;
import java.util.ArrayList;
import java.lang.String;
import java.util.function.Predicate;
import java.lang.StringBuffer;
import java.lang.Deprecated;
import java.util.Iterator;
import java.util.Collection;
import java.lang.Object;
import java.util.List;
import io.dekorate.kubernetes.annotation.AccessMode;
import java.lang.Boolean;

public class PersistentVolumeClaimConfigFluentImpl<A extends PersistentVolumeClaimConfigFluent<A>> extends ApplicationConfigurationFluentImpl<A> implements PersistentVolumeClaimConfigFluent<A> {

    private int size;
    private String unit;
    private String storageClass;
    private AccessMode accessMode;
    private List<LabelBuilder> matchLabels =  new ArrayList<LabelBuilder>();

    public PersistentVolumeClaimConfigFluentImpl() {
    }

    public PersistentVolumeClaimConfigFluentImpl(PersistentVolumeClaimConfig instance) {
        this.withProject(instance.getProject()); 
        this.withAttributes(instance.getAttributes()); 
        this.withPartOf(instance.getPartOf()); 
        this.withName(instance.getName()); 
        this.withVersion(instance.getVersion()); 
        this.withSize(instance.getSize()); 
        this.withUnit(instance.getUnit()); 
        this.withStorageClass(instance.getStorageClass()); 
        this.withAccessMode(instance.getAccessMode()); 
        this.withMatchLabels(instance.getMatchLabels()); 
    }

    public int getSize() {
        return this.size;
    }

    public A withSize(int size) {
        this.size=size; return (A) this;
    }

    public Boolean hasSize() {
        return true;
    }

    public String getUnit() {
        return this.unit;
    }

    public A withUnit(String unit) {
        this.unit=unit; return (A) this;
    }

    public Boolean hasUnit() {
        return this.unit != null;
    }

    public A withNewUnit(String arg1) {
        return (A)withUnit(new String(arg1));
    }

    public A withNewUnit(StringBuilder arg1) {
        return (A)withUnit(new String(arg1));
    }

    public A withNewUnit(StringBuffer arg1) {
        return (A)withUnit(new String(arg1));
    }

    public String getStorageClass() {
        return this.storageClass;
    }

    public A withStorageClass(String storageClass) {
        this.storageClass=storageClass; return (A) this;
    }

    public Boolean hasStorageClass() {
        return this.storageClass != null;
    }

    public A withNewStorageClass(String arg1) {
        return (A)withStorageClass(new String(arg1));
    }

    public A withNewStorageClass(StringBuilder arg1) {
        return (A)withStorageClass(new String(arg1));
    }

    public A withNewStorageClass(StringBuffer arg1) {
        return (A)withStorageClass(new String(arg1));
    }

    public AccessMode getAccessMode() {
        return this.accessMode;
    }

    public A withAccessMode(AccessMode accessMode) {
        this.accessMode=accessMode; return (A) this;
    }

    public Boolean hasAccessMode() {
        return this.accessMode != null;
    }

    public A withMatchLabels(io.dekorate.kubernetes.config.Label... matchLabels) {
        if (this.matchLabels != null) {this.matchLabels.clear();}
        if (matchLabels != null) {for (io.dekorate.kubernetes.config.Label item :matchLabels){ this.addToMatchLabels(item);}} return (A) this;
    }

    
/**
 * This method has been deprecated, please use method buildMatchLabels instead.
 */
@Deprecated public io.dekorate.kubernetes.config.Label[] getMatchLabels() {
        int size = matchLabels != null ? matchLabels.size() : 0;
        io.dekorate.kubernetes.config.Label[] result = new io.dekorate.kubernetes.config.Label[size];
        if (size == 0) {
           return result;
        }
        int index = 0;
        for (VisitableBuilder<? extends io.dekorate.kubernetes.config.Label,?> builder : matchLabels) {
            result[index++]=builder.build();
        }
        return result;
    }

    public io.dekorate.kubernetes.config.Label[] buildMatchLabels() {
        int size = matchLabels != null ? matchLabels.size() : 0;
        io.dekorate.kubernetes.config.Label[] result = new io.dekorate.kubernetes.config.Label[size];
        if (size == 0) {
           return result;
        }
        int index = 0;
        for (VisitableBuilder<? extends io.dekorate.kubernetes.config.Label,?> builder : matchLabels) {
            result[index++]=builder.build();
        }
        return result;
    }

    public io.dekorate.kubernetes.config.Label buildMatchLabel(int index) {
        return this.matchLabels.get(index).build();
    }

    public io.dekorate.kubernetes.config.Label buildFirstMatchLabel() {
        return this.matchLabels.get(0).build();
    }

    public io.dekorate.kubernetes.config.Label buildLastMatchLabel() {
        return this.matchLabels.get(matchLabels.size() - 1).build();
    }

    public io.dekorate.kubernetes.config.Label buildMatchingMatchLabel(Predicate<LabelBuilder> predicate) {
        for (LabelBuilder item: matchLabels) { if(predicate.test(item)){ return item.build();} } return null;
    }

    public Boolean hasMatchingMatchLabel(Predicate<LabelBuilder> predicate) {
        for (LabelBuilder item: matchLabels) { if(predicate.test(item)){ return true;} } return false;
    }

    public A addToMatchLabels(int index,io.dekorate.kubernetes.config.Label item) {
        if (this.matchLabels == null) {this.matchLabels = new ArrayList<LabelBuilder>();}
        io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);_visitables.get("matchLabels").add(index >= 0 ? index : _visitables.get("matchLabels").size(), builder);this.matchLabels.add(index >= 0 ? index : matchLabels.size(), builder); return (A)this;
    }

    public A setToMatchLabels(int index,io.dekorate.kubernetes.config.Label item) {
        if (this.matchLabels == null) {this.matchLabels = new ArrayList<LabelBuilder>();}
        io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);
        if (index < 0 || index >= _visitables.get("matchLabels").size()) { _visitables.get("matchLabels").add(builder); } else { _visitables.get("matchLabels").set(index, builder);}
        if (index < 0 || index >= matchLabels.size()) { matchLabels.add(builder); } else { matchLabels.set(index, builder);}
         return (A)this;
    }

    public A addToMatchLabels(io.dekorate.kubernetes.config.Label... items) {
        if (this.matchLabels == null) {this.matchLabels = new ArrayList<LabelBuilder>();}
        for (io.dekorate.kubernetes.config.Label item : items) {io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);_visitables.get("matchLabels").add(builder);this.matchLabels.add(builder);} return (A)this;
    }

    public A addAllToMatchLabels(Collection<io.dekorate.kubernetes.config.Label> items) {
        if (this.matchLabels == null) {this.matchLabels = new ArrayList<LabelBuilder>();}
        for (io.dekorate.kubernetes.config.Label item : items) {io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);_visitables.get("matchLabels").add(builder);this.matchLabels.add(builder);} return (A)this;
    }

    public A removeFromMatchLabels(io.dekorate.kubernetes.config.Label... items) {
        for (io.dekorate.kubernetes.config.Label item : items) {io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);_visitables.get("matchLabels").remove(builder);if (this.matchLabels != null) {this.matchLabels.remove(builder);}} return (A)this;
    }

    public A removeAllFromMatchLabels(Collection<io.dekorate.kubernetes.config.Label> items) {
        for (io.dekorate.kubernetes.config.Label item : items) {io.dekorate.kubernetes.config.LabelBuilder builder = new io.dekorate.kubernetes.config.LabelBuilder(item);_visitables.get("matchLabels").remove(builder);if (this.matchLabels != null) {this.matchLabels.remove(builder);}} return (A)this;
    }

    public A removeMatchingFromMatchLabels(Predicate<LabelBuilder> predicate) {
        if (matchLabels == null) return (A) this;
        final Iterator<LabelBuilder> each = matchLabels.iterator();
        final List visitables = _visitables.get("matchLabels");
        while (each.hasNext()) {
          LabelBuilder builder = each.next();
          if (predicate.test(builder)) {
            visitables.remove(builder);
            each.remove();
          }
        }
        return (A)this;
    }

    public Boolean hasMatchLabels() {
        return matchLabels != null && !matchLabels.isEmpty();
    }

    public A addNewMatchLabel(String key,String value,String[] kinds) {
        return (A)addToMatchLabels(new Label(key, value, kinds));
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> addNewMatchLabel() {
        return new MatchLabelsNestedImpl();
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> addNewMatchLabelLike(io.dekorate.kubernetes.config.Label item) {
        return new MatchLabelsNestedImpl(-1, item);
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> setNewMatchLabelLike(int index,io.dekorate.kubernetes.config.Label item) {
        return new MatchLabelsNestedImpl(index, item);
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> editMatchLabel(int index) {
        if (matchLabels.size() <= index) throw new RuntimeException("Can't edit matchLabels. Index exceeds size.");
        return setNewMatchLabelLike(index, buildMatchLabel(index));
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> editFirstMatchLabel() {
        if (matchLabels.size() == 0) throw new RuntimeException("Can't edit first matchLabels. The list is empty.");
        return setNewMatchLabelLike(0, buildMatchLabel(0));
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> editLastMatchLabel() {
        int index = matchLabels.size() - 1;
        if (index < 0) throw new RuntimeException("Can't edit last matchLabels. The list is empty.");
        return setNewMatchLabelLike(index, buildMatchLabel(index));
    }

    public PersistentVolumeClaimConfigFluent.MatchLabelsNested<A> editMatchingMatchLabel(Predicate<LabelBuilder> predicate) {
        int index = -1;
        for (int i=0;i<matchLabels.size();i++) { 
        if (predicate.test(matchLabels.get(i))) {index = i; break;}
        } 
        if (index < 0) throw new RuntimeException("Can't edit matching matchLabels. No match found.");
        return setNewMatchLabelLike(index, buildMatchLabel(index));
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        if (!super.equals(o)) return false;
        PersistentVolumeClaimConfigFluentImpl that = (PersistentVolumeClaimConfigFluentImpl) o;
        if (size != that.size) return false;
        if (unit != null ? !unit.equals(that.unit) :that.unit != null) return false;
        if (storageClass != null ? !storageClass.equals(that.storageClass) :that.storageClass != null) return false;
        if (accessMode != null ? !accessMode.equals(that.accessMode) :that.accessMode != null) return false;
        if (matchLabels != null ? !matchLabels.equals(that.matchLabels) :that.matchLabels != null) return false;
        return true;
    }

    public int hashCode() {
        return java.util.Objects.hash(size,  unit,  storageClass,  accessMode,  matchLabels,  super.hashCode());
    }

    public class MatchLabelsNestedImpl<N> extends LabelFluentImpl<PersistentVolumeClaimConfigFluent.MatchLabelsNested<N>> implements PersistentVolumeClaimConfigFluent.MatchLabelsNested<N>,io.fabric8.kubernetes.api.builder.Nested<N> {
        private final LabelBuilder builder;
        private final int index;

            MatchLabelsNestedImpl(int index,io.dekorate.kubernetes.config.Label item) {
                this.index = index;
                this.builder = new LabelBuilder(this, item);
                        
            }

            MatchLabelsNestedImpl() {
                this.index = -1;
                this.builder = new LabelBuilder(this);
                        
            }

            public N and() {
                return (N) PersistentVolumeClaimConfigFluentImpl.this.setToMatchLabels(index,builder.build());
            }

            public N endMatchLabel() {
                return and();
            }
    }


}
